<?php defined('SYSPATH') or die('No direct script access.');
/*
 * This file is part of open source system FreenetIS
 * and it is release under GPLv3 licence.
 * 
 * More info about licence can be found:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * 
 * More info about project can be found:
 * http://www.freenetis.org/
 * 
 */

/**
 * Vote
 * 
 * @package Model
 * 
 * @property integer $user_id
 * @property User_Model $user
 * @property integer $type
 * @property integer $fk_id
 * @property integer $aro_group_id
 * @property integer $priority
 * @property integer $this
 * @property datetime $time
 * @property string $comment
 */
class Vote_Model extends ORM
{
	/** Type of vote for a work */
	const WORK	= 1;
	
	/** Type of vote for work report */
	const WORK_REPORT = 2;
	
	/** Type of vote to request */
	const REQUEST	= 3;
	
	/** Types of vote **/
	
	// abstain
	const ABSTAIN	= 0;
	
	// agree
	const AGREE	= 1;
	
	// disagree
	const DISAGREE	= -1;
	
	// new item, one votes about it, can be edited
	const STATE_NEW = 0;
	
	// open item, approval already started, cannot be edited
	const STATE_OPEN = 1;
	
	// rejected item
	const STATE_REJECTED = 2;
	
	// approved item
	const STATE_APPROVED = 3;
	
	/**
	 * Vote options
	 *
	 * @var array
	 */
	private static $vote_options = array
	(
		self::AGREE	=> 'Agree',
		self::DISAGREE	=> 'Disagree',
		self::ABSTAIN	=> 'Abstain',
	);
	
	// state names
	private static $states = array
	(
		self::STATE_NEW		=> 'New',
		self::STATE_OPEN	=> 'Open',
		self::STATE_REJECTED	=> 'Rejected',
		self::STATE_APPROVED	=> 'Approved'
	);
	
	// state colors
	private static $state_colors = array
	(
		self::STATE_NEW		=> 'black',
		self::STATE_OPEN	=> 'black',
		self::STATE_REJECTED	=> 'red',
		self::STATE_APPROVED	=> 'green'
	);
	
	protected $belongs_to = array('user');
	
	/**
	 * Returns vote options
	 * 
	 * @author Michal Kliment
	 * @param type $type Type of item to vote
	 * @param type $is_own Whether vote about own item (e.g. work)
	 * @return array Vote options
	 */
	public static function get_vote_options($type = NULL, $is_own = NULL)
	{
		$options = array();
		
		foreach (self::$vote_options as $key => $value)
		{
			if ($type == self::WORK && $is_own && $key != self::ABSTAIN)
				continue;
			
			$options[$key] = __($value);
		}
		
		return $options;
	}
	
	/**
	 * Returns vote option name
	 * 
	 * @author Michal Kliment
	 * @param integer $option
	 * @return string
	 */
	public static function get_vote_option_name ($option)
	{
		if (array_key_exists($option, self::$vote_options))
			return __(self::$vote_options[$option]);
		else
			return '';
	}
	
	/**
	 * Returns all states
	 * 
	 * @author Michal Kliment
	 * @return array
	 */
	public static function get_states()
	{
		$states = array();
		
		foreach (self::$states as $key => $value)
		{
			$states[$key] = __($value);
		}
		
		return $states;
	}
	
	/**
	 * Returns state name
	 * 
	 * @author Michal Kliment
	 * @param integer $option
	 * @return string
	 */
	public static function get_state_name ($state)
	{
		if (array_key_exists($state, self::$states))
			return __(self::$states[$state]);
		else
			return '';
	}
	
	/**
	 * Returns state color
	 * 
	 * @author Michal Kliment
	 * @param integer $option
	 * @return string
	 */
	public static function get_state_color ($state)
	{
		if (array_key_exists($state, self::$state_colors))
			return __(self::$state_colors[$state]);
		else
			return '';
	}


	/**
	 * Get all wotes by work
	 * 
	 * @param integer $work_id 
	 * @return Mysql_Result
	 */
	public function get_all_votes_by_work($work_id)
	{
		return $this->db->query("
				SELECT * FROM votes v
				WHERE v.type = 1 AND v.fk_id = ?
		", $work_id);
	}
	
	/**
	 * Check if user voted about object
	 *
	 * @param integer $user_id	User's ID
	 * @param integer $fk_id	ID of object
	 * @return boolean
	 */
	public function has_user_voted_about($user_id, $fk_id)
	{
		return $this->db->query("
				SELECT COUNT(*) AS count
				FROM votes
				WHERE user_id = ? AND fk_id = ?
		", $user_id, $fk_id)->current()->count > 0;
	}
	
	/**
	 * Function to get state of work approval from all votes
	 * by approval template and type.
	 * 
	 * @author Michal Kliment, Ondřej Fibich
	 * @param integer $type
	 * @return integer
	 */
	public static function get_state($object, $approval_type_id = NULL)
	{
		if (!$object->id)
		{
			throw new Exception('Invalid call of method, object has to be loaded.');
		}
		
		if ($object instanceof Job_Model || $object instanceof Job_report_Model)
		{
			$type = self::WORK;
		}
		else if ($object instanceof Request_Model)
		{
			$type = self::REQUEST;
		}
		else
		{
			throw new Exception('Invalid loaded object: '.get_class($object));
		}
		
		$approval_template_item_model = new Approval_template_item_Model();
		$groups_aro_map_model = new Groups_aro_map_Model();
		$vote_model = new Vote_Model();
		
		$votes_count = $vote_model->where('type', $type)
				->where('fk_id', $object->id)
				->count_all();

		// 0 votes - vote not yet started
		if (!$votes_count)
		{
			$state = 0;
			return $state;
		}

		if (!empty($approval_type_id))
		{
			$approval_template_items = $approval_template_item_model
					->where('approval_template_id', $object->approval_template_id)
					->where('approval_type_id', $approval_type_id)
					->orderby('priority', 'desc')
					->find_all();
		}
		else
		{
			$approval_template_items = $approval_template_item_model
					->where('approval_template_id', $object->approval_template_id)
					->orderby('priority', 'desc')
					->find_all();
		}
		
		$arr_users = array();
		$state = 3;
		
		$suggest_amount = $object->get_suggest_amount();

		foreach ($approval_template_items as $approval_template_item)
		{
			$at = $approval_template_item->approval_type;
			
			if (!$suggest_amount || $at->min_suggest_amount <= $suggest_amount)
			{
				$count = 0;
				$total_votes = 0;
				$agree = 0;
				$abstain = 0;
				
				$users = $groups_aro_map_model->get_all_users_by_group_id(
						$at->aro_group_id
				);
				
				foreach ($users as $user)
				{
					if (in_array($user->id, $arr_users))
					{
						continue;
					}
					
					$arr_users[] = $user->id;
					$count++;

					$vote = $vote_model->where('user_id', $user->id)
							->where('type', $type)
							->where('fk_id', $object->id)
							->find();

					if (!$vote || !$vote->id)
					{
						$interval = date::hour_diff($at->interval);

						if (!$interval)
							$state = 1;
					}
					else
					{
						$total_votes++;

						if ($at->type == Approval_type_Model::SIMPLE &&
							$vote->vote == 0)
						{
							$abstain++;
						}

						if ($vote->vote == 1)
							$agree++;
					}
				}

				if (!$count)
					continue;

				if ($count == $total_votes)
				{
					$total_votes -= $abstain;

					$percent = ($total_votes) ? round($agree/$total_votes*100, 2) : 0;

					if ($percent < $at->majority_percent)
					{
						$state = 2;
						return $state;
					}
				}
				else
				{
					$interval = date::hour_diff($at->interval);
					if ($interval)
					{
						$count -= $abstain;

						$percent = ($count) ? round($agree/$count*100, 2) : 0;

						if ($percent < $at->majority_percent)
						{
							$agree += ($count - $total_votes + $abstain);

							$percent = ($count) ? round($agree/$count*100, 2) : 0;

							if ($percent < $at->majority_percent)
							{
								$state = 2;
								return $state;
							}
							else
								$state = 1;
						}
					}
					else
						$state = 1;
				}
			}
		}
		
		return $state;
	}
	
	/**
	 * Returns all items (works, work reports and requests) to which user can vote
	 * 
	 * @author Michal Kliment
	 * @param integer $user_id
	 * @return array
	 */
	public function get_all_items_user_can_vote($user_id)
	{	
		$items = $this->db->query("
			SELECT i.*
			FROM groups_aro_map gam
			JOIN approval_types at ON at.aro_group_id = gam.group_id
			JOIN approval_template_items ati ON ati.approval_type_id = at.id
			JOIN
			(
				SELECT
					? AS type,
					j.id AS fk_id,
					suggest_amount,
					approval_template_id
				FROM jobs j
				WHERE state = ? OR state = ?
				UNION
				SELECT
					? AS type,
					r.id AS fk_id,
					suggest_amount,
					approval_template_id
				FROM requests r
				WHERE state = ? OR state = ?
			) i
			ON i.approval_template_id = ati.approval_template_id AND
				(i.suggest_amount = 0 OR i.suggest_amount >= at.min_suggest_amount)
			WHERE gam.aro_id = ?
		", array
		(
			self::WORK,
			self::STATE_NEW,
			self::STATE_OPEN,
			self::REQUEST,
			self::STATE_NEW,
			self::STATE_OPEN,
			$user_id
		));
		
		$result = array();
		
		foreach ($items as $item)
		{
			if (!array_key_exists($item->type, $result))
				$result[$item->type] = array();
			
			$result[$item->type][] = $item->fk_id;
		}
		
		return $result;
	}
	
	/**
	 * Inserts new vote
	 * 
	 * @author Michal Kliment
	 * @param integer $user_id
	 * @param integer $type
	 * @param integer $fk_id
	 * @param integer $vote
	 * @param string $comment
	 * @param integer $aro_group_id
	 * @param datetime $time
	 * @return Vote_Model
	 */
	public static function insert (
		$user_id, $type, $fk_id, $vote, $comment, $aro_group_id,
		$time = NULL)
	{
		if (!$time)
			$time = date('Y-m-d H:i:s');
		
		$vote_model = new Vote_Model();
		$vote_model->user_id		= $user_id;
		$vote_model->type		= $type;
		$vote_model->fk_id		= $fk_id;
		$vote_model->aro_group_id	= $aro_group_id;
		$vote_model->vote		= $vote;
		$vote_model->time		= $time;
		$vote_model->comment		= $comment;
		$vote_model->save_throwable();
		
		return $vote_model;
	}
	
	/**
	 * Removes vote
	 * 
	 * @author Michal Kliment
	 * @param integer $user_id
	 * @param integer $type
	 * @param integer $fk_id
	 * @return type
	 */
	public function remove_vote ($user_id, $type, $fk_id)
	{
		return $this->db->query("
			DELETE FROM votes
			WHERE user_id = ? AND type = ? AND fk_id = ?
		", array($user_id, $type, $fk_id));
	}
}
